#!/usr/bin/env python
"""
Created on 16 Jul 2022

@author: David Doblas Jiménez
@email: daviddoji@pm.me

Solution for problem 061 of Project Euler
https://projecteuler.net/problem=61
"""

from itertools import permutations

from utils import timeit


def get_numbers(order: int, number: int) -> int:
    return (number * (number - 1) * (order - 2) // 2) + number


poligonals: dict[int, set[str]] = {
    o: set(str(get_numbers(o, n)) for n in range(150)) for o in range(3, 9)
}

four_digits_numbers: set[str] = {str(n) for n in range(10**3, 10**4)}

eligibles: dict[int, list[str]] = {
    k: list(v & four_digits_numbers) for k, v in poligonals.items()
}


@timeit("Problem 061")
def compute():
    """
    Triangle, square, pentagonal, hexagonal, heptagonal, and octagonal numbers
    are all figurate (polygonal) numbers and are generated by the following
    formulae:
        Triangle 	  	P3,n=n(n+1)/2 	  	1, 3, 6, 10, 15, ...
        Square 	  	    P4,n=n2 	  	    1, 4, 9, 16, 25, ...
        Pentagonal 	  	P5,n=n(3n-1)/2 	  	1, 5, 12, 22, 35, ...
        Hexagonal 	  	P6,n=n(2n-1) 	  	1, 6, 15, 28, 45, ...
        Heptagonal 	  	P7,n=n(5n-3)/2 	  	1, 7, 18, 34, 55, ...
        Octagonal 	  	P8,n=n(3n-2) 	  	1, 8, 21, 40, 65, ...

    The ordered set of three 4-digit numbers: 8128, 2882, 8281, has three
    interesting properties.

        The set is cyclic, in that the last two digits of each number is the
        first two digits of the next number (including the last number with the
        first).
        Each polygonal type: triangle (P3,127=8128), square (P4,91=8281), and
        pentagonal (P5,44=2882), is represented by a different number in the
        set.
        This is the only set of 4-digit numbers with this property.

    Find the sum of the only ordered set of six cyclic 4-digit numbers for
    which each polygonal type: triangle, square, pentagonal, hexagonal,
    heptagonal, and octagonal, is represented by a different number in the set.
    """

    perms = permutations(eligibles.keys(), 6)
    for perm in perms:
        for a in eligibles.get(perm[0]):
            ans = []
            a1, a2 = a[:2], a[2:]
            for b in filter(lambda n: n[:2] == a2, eligibles.get(perm[1])):
                b2 = b[2:]
                for c in filter(lambda n: n[:2] == b2, eligibles.get(perm[2])):
                    c2 = c[2:]
                    for d in filter(lambda n: n[:2] == c2, eligibles.get(perm[3])):
                        d2 = d[2:]
                        for e in filter(lambda n: n[:2] == d2, eligibles.get(perm[4])):
                            e2 = e[2:]
                            for f in filter(
                                lambda n: n[:2] == e2 and n[2:] == a1,
                                eligibles.get(perm[5]),
                            ):
                                ans.append([a, b, c, d, e, f])
                                return sum(map(int, ans[0]))


if __name__ == "__main__":
    print(f"Result for Problem 061: {compute()}")
