#!/usr/bin/env python
"""
Created on 08 Sep 2023

@author: David Doblas Jiménez
@email: daviddoji@pm.me

Solution for problem 066 from Project Euler
https://projecteuler.net/problem=66
"""

from sympy import simplify
from sympy.abc import t, x, y
from sympy.solvers.diophantine.diophantine import diop_quadratic
from utils import timeit


@timeit("Problem 066")
def compute():
    """
    Consider quadratic Diophantine equations of the form:

        x^2 - Dy^2 = 1

    For example, when D = 13, the minimal solution in x is
    649^2 - 13 * 180^2 = 1

    It can be assumed that there are no solutions in positive integers when D is
    square.

    By finding minimal solutions in x for D = {2, 3, 5, 6, 7}, we obtain the
    following:

        3^2 - 2 * 2^2 = 1
        2^2 - 3 * 1^2 = 1
        9^2 - 5 * 4^2 = 1
        5^2 - 6 * 2^2 = 1
        8^2 - 7 * 3^2 = 1

    Hence, by considering minimal solutions in x for D <= 7, the largest x is
    obtained when D = 5.

    Find the value of D <= 1000 in minimal solutions of x for which the largest
    value of x is obtained.
    """

    max_d, max_x = 0, 0
    for d in range(2, 1000):
        solve = diop_quadratic(x**2 - d * y**2 - 1, t)
        for i in solve:
            sol = simplify(i.subs({t: 0}))
            xx = sol[0]
            if xx > max_x:
                max_x = xx
                max_d = d

    return max_d


if __name__ == "__main__":
    print(f"Result for Problem 066 is {compute()}")
